@@ -5,15 +5,17 @@
 
 =head1 NAME
 
-Set::Infinite::Date - a 'date' scalar
+Set::Infinite::Date - 'date' scalar - Deprecated - use Date::Set instead
 
 =head1 SYNOPSIS
 
+This module is obsolete - use Date::Set instead
+
 	use Set::Infinite::Date;
 
 	$a = Set::Infinite::Date->new("10:00");
 
-	This module requires HTTP:Date and Time::Local
+	This module requires Time::Local
 
 =head1 USAGE
 
@@ -44,7 +46,7 @@ Date input format:
 
 String conversion functions:
 
-	0 + $s	returns the Date as a time-number. 
+	0 + $s	returns the Date as a time-number (epoch).
 			This is faster than	date2time or hour2time.
 
 	time2date 
@@ -64,8 +66,9 @@ String conversion functions:
 Internal functions:
 	$a->mode($b);
 		mode can be 
-		1 (beginning in 00:00:00) or 
-		2 (absolute dates like 2001-01-01 00:00:00)
+		0 - epoch
+		1 - beginning in 00:00:00 
+		2 - absolute dates like 2001-01-01 00:00:00
 
 =head1 TODO
 
@@ -80,10 +83,8 @@ Internal functions:
 
 require Exporter;
 package Set::Infinite::Date;
-$VERSION = "0.17";
 
 my $DEBUG = 1;
-my $package = 'Set::Infinite::Date';
 # @ISA = qw(Set::Infinite::Simple); # DON'T !
 @ISA = qw(Set::Infinite::Element_Inf);  # is_null
 @EXPORT = qw();
@@ -92,12 +93,35 @@ my $package = 'Set::Infinite::Date';
 );
 
 use strict;
-use HTTP::Date qw(str2time);
 use Time::Local;
+use Set::Infinite::Element_Inf;
+
+#--- THIS FUNCTION IS (HEAVILY) MODIFIED FROM HTTP::Date -- Copyright 1995-1999, Gisle Aas
+
+sub str2time {
+    my $str = shift;    # '1996-02-29 12:00:00' 
+    my @d = $str =~ 
+	/(\d{4})-(\d\d?)-(\d\d?)
+     (?:
+	       (?:\s+)  # separator before clock
+	    (\d\d?):?(\d\d)    # hour:min
+	    (?::?(\d\d(?:\.\d*)?))?  # optional seconds (and fractional)
+	 )?                    # optional clock
+	/x; 
+    $d[0] -= 1900;  # year
+    $d[1]--;        # month
+    $d[3] = 0 unless $d[3];
+    $d[4] = 0 unless $d[4];
+    $d[5] = 0 unless $d[5];
+	# print " # $str == ", join(";",@d), " \n";
+    return timegm(reverse @d);
+}
+
+#--- END CODE DERIVED FROM HTTP::Date
 
 use overload
+	'0+' => sub { return $_[0]->{a} },
 	'<=>' => \&spaceship,
-	'cmp' => \&cmp,
 	'-' => \&sub,
 	'+' => \&add,
 	'/' => \&div,
@@ -107,13 +131,16 @@ our $quantizer = 'Set::Infinite::Quantize_Date';
 our $selector  = 'Set::Infinite::Select';
 our $offsetter = 'Set::Infinite::Offset';
 
-our $day_size = timelocal(0,0,0,2,3,2001) - timelocal(0,0,0,1,3,2001);
+our $day_size = timegm(0,0,0,2,3,2001) - timegm(0,0,0,1,3,2001);
 our $hour_size = $day_size / 24;
 our $minute_size = $hour_size / 60;
 our $second_size = $minute_size / 60;
 
 our $date_format = "year-month-day hour:min:sec";
 
+our %date_cache = ();
+our %time2date_cache = ();
+
 sub day_size { $day_size }
 
 sub quantizer {
@@ -125,29 +152,39 @@ sub quantizer {
 
 sub date_format {
 	$date_format = pop if @_;
+	# %date_cache = ();
+	%time2date_cache = ();
 	return $date_format;
 }
 
 # export time2date and date2time
+
 sub time2date (;$)  { 
-	my $tmp = shift;
-	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($tmp);
+	my $a = shift;
+	if (exists $time2date_cache{$a}) { 
+		# print "*"; 
+		return $time2date_cache{$a} 
+	};
+	# print " [ttd:$a] ";
+
+	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = gmtime($a);
 	$year += 1900;
 	$mon++;
 	foreach ($sec,$min,$hour,$mday,$mon,$year) {
 		$_ = '0' . $_ if $_ < 10;
 	}
-	$tmp = $date_format;
+	my $tmp = $date_format;
 	$tmp =~ s/year/$year/;
 	$tmp =~ s/month/$mon/;
 	$tmp =~ s/day/$mday/;
 	$tmp =~ s/sec/$sec/;
 	$tmp =~ s/min/$min/;
 	$tmp =~ s/hour/$hour/;
+	$time2date_cache{$a} = $tmp;
 	return $tmp;
 }
 
-sub date2time ($;$) { return HTTP::Date::str2time (shift) }
+sub date2time ($;$) { return str2time (shift) }
 
 sub time2hour {
 	my $a = shift;
@@ -177,20 +214,24 @@ sub div {
 sub sub {
 	# TODO: (since 0.21) keep format ("mode") in result (see: "Date::add")
 	my ($tmp1, $tmp2, $inverted) = @_;
-	$tmp2 = Set::Infinite::Date->new($tmp2) unless ref($tmp2) and $tmp2->isa(__PACKAGE__);
+	my $result;
+	$tmp2 = Set::Infinite::Date->new($tmp2) unless ref($tmp2) and (ref($tmp2) eq __PACKAGE__);
 	if ($inverted) {
 		return - $tmp1 if $tmp2->is_null;
-		return Set::Infinite::Date->new( $tmp2->{a} - $tmp1->{a} );
+		$result = Set::Infinite::Date->new( $tmp2->{a} - $tmp1->{a} );
+		$result->{mode} = $tmp2->{mode};
 	}
 	return $tmp1 unless defined($tmp2);
-	return Set::Infinite::Date->new( $tmp1->{a} - $tmp2->{a} );
+	$result = Set::Infinite::Date->new( $tmp1->{a} - $tmp2->{a} );
+	$result->{mode} = $tmp1->{mode};
+	return $result;
 }
 
 sub add {
 	my ($tmp1, $tmp2, $inverted) = @_;
 	return $tmp1 unless defined($tmp2);
 	# print " [DATE::ADD $tmp1 $tmp2] ";
-	$tmp2 = Set::Infinite::Date->new($tmp2) unless ref($tmp2) and $tmp2->isa(__PACKAGE__);
+	$tmp2 = Set::Infinite::Date->new($tmp2) unless ref($tmp2) eq __PACKAGE__;
 	my $result = Set::Infinite::Date->new( $tmp1->{a} + $tmp2->{a} );
 	if ($inverted) {
 		$result->{mode} = $tmp2->{mode};
@@ -203,56 +244,61 @@ sub add {
 
 sub spaceship {
 	my ($tmp1, $tmp2, $inverted) = @_;
-	# print " [DATE:CMP:",caller(1),"]\n";
-	# print " $tmp1 ";
 
 	return 1 unless defined($tmp2);
 
-	# if ($DEBUG) {
-		# if ($tmp2->isa(__PACKAGE__)) {
-		#	print " [DATE:CMP:$inverted:",$tmp1->{a};
-		#	print " [ ",$tmp2,": ", ref($tmp2), ": ", ref(\$tmp2), "]\n";
-		#	print "    <=>",$tmp2->{a},"]\n";
-		#}
-		# { else {
-		#	print " [DATE:CMP:$inverted:",$tmp1;
-		#	print " [ ",$tmp2,": ", ref($tmp2), ": ", ref(\$tmp2), "]\n";
-		#	print "    <=>",$tmp2,"]\n";
-		# }
-	# }
-
-	$tmp2 = Set::Infinite::Date->new($tmp2) unless ref($tmp2) and $tmp2->isa(__PACKAGE__);
+	if ( ref($tmp2) eq __PACKAGE__ ) {
+		if ($inverted) {
+			return ( $tmp2->{a} <=> $tmp1->{a} );
+		}
+		return ( $tmp1->{a} <=> $tmp2->{a} );
+	}
 
 	if ($inverted) {
-		return ( $tmp2->{a} <=> $tmp1->{a} );
+		return ( $tmp2 <=> $tmp1->{a} );
 	}
-	return ( $tmp1->{a} <=> $tmp2->{a} );
-}
-
-sub cmp {
-	return spaceship @_;
+	return ( $tmp1->{a} <=> $tmp2 );
 }
 
 sub new {
-	my ($self) = bless {}, shift;
+	my $class = shift;
+	my $self;
+	if (ref($class)) {
+		$self = bless { mode => $class->{mode} }, ref($class);
+	}
+	else {
+		$self = bless {}, $class;
+	}
 	my $tmp = shift;
+
+	if ((not defined $tmp) or ($tmp eq '')) {
+		# print " [date:new:null] ";
+		return Set::Infinite::Element_Inf->null ;
+	}
+
+	if (exists $date_cache{$tmp}) {
+		# print "*";
+		return $date_cache{$tmp};
+	}
+	# print "N";
 	$tmp = '' unless defined $tmp;
 	$self->{string} = '';
 	if ($tmp =~ /\d[\/\.\-]\d/) {
 		# $self->{string} = $tmp;
 		$self->{a} = date2time($tmp);
-		$self->{mode} = 2;
+		$self->{mode} = 2 unless $self->{mode};
 	}
 	elsif ($tmp =~ /\d\:\d/) {
 		# $self->{string} = $tmp;
 		$self->{a} = hour2time($tmp);
-		$self->{mode} = 1;
+		$self->{mode} = 1 unless $self->{mode};
 	}
 	else {
 		# $self->{string} = $tmp;
 		$self->{a} = $tmp;
-		$self->{mode} = 0;
+		$self->{mode} = 0 unless $self->{mode};
 	}
+	$date_cache{$tmp} = $self;
 	return $self;
 }
 
@@ -268,6 +314,7 @@ sub as_string {
 
 	#return '' unless defined($self);
 	#print " [ $self->{string} ] " ;
+	# $self->{string} = '';
 
 	if ($self->{string} ne '') {
 		# done
@@ -290,26 +337,4 @@ sub as_string {
 	return $self->{string};
 }
 
-# TIE
-
-sub TIESCALAR {
-	my $class = shift;
-	my $self = $class->new(@_);
-	return $self;
-}
-
-sub FETCH {
-	my ($self) = shift;
-	return $self->as_string;
-}
-
-sub STORE {
-	my ($self) = shift;
-	$self = new(@_);
-	return @_;
-}
-
-sub DESTROY {
-}
-
-1;
\ No newline at end of file
+1;
